#
# $Id: hammer.py,v 1.25 2001/10/29 07:52:53 mrnolta Exp $
#
# Copyright (C) 2000-2001 :
#
#   Walter Brisken <walterfb@users.sourceforge.net>
#   Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

#
# Hammer-Aitoff coordinates are an equal area projection of the
# sphere into the plane. The spherical coordinates l and b are
# used where l runs from -pi to pi and b from -pi/2 to pi/2.
# The equator is b=0, and b = +/-pi/2 are the north/south poles.
#

import math
from biggles import \
	_series, _PlotComposite, _PlotGeometry, _PlotContainer, Geodesic, Curve
from geometry import *
import _biggles

class _HammerAitoffGeometry:

	def __init__( self, dest, l0=0., b0=0, rot=0. ):
		self.src_bbox = BoundingBox( (-1.,-.5), (1.,.5) )
		self.dest_bbox = dest
		self.aff = RectilinearMap( self.src_bbox, dest )
		self.l0 = l0
		self.b0 = b0
		self.rot = rot

	def __call__( self, l_, b_ ):
		xh, yh = _biggles.hammer_call( \
			l_, b_, self.l0, self.b0, self.rot )
		return self.aff( xh, yh )

	def call_vec( self, l_, b_ ):
		xh, yh = _biggles.hammer_call_vec( \
			l_, b_, self.l0, self.b0, self.rot )
		return self.aff.call_vec( xh, yh )

	def geodesic( self, l_, b_, div=2 ):
		l, b = _biggles.hammer_geodesic_fill( l_, b_, div )
		segs = []
		i0 = 0
		for i in range(1,len(l)):
			if _biggles.hammer_connect( \
					l[i-1], b[i-1], l[i], b[i], \
					self.l0, self.b0, self.rot ):
				segs.append( (l[i0:i],b[i0:i]) )
				i0 = i
		segs.append( (l[i0:],b[i0:]) )
		return segs

class _HammerAitoffContext:

	def __init__( self, device, dev, l0=0., b0=0., rot=0. ):
		self.draw = device
		self.dev_bbox = dev
		self.geom = _HammerAitoffGeometry( dev, l0, b0, rot )
		self.plot_geom = _PlotGeometry( BoundingBox((0,0),(1,1)), dev )

	def do_clip( self ):
		pass

class HammerAitoffPlot( _PlotContainer ):

	_attr_deprecated = {
		"num_l_ribs"	: "ribs_l",
		"num_b_ribs"	: "ribs_b",
	}

	def __init__( self, l0=0., b0=0, rot=0., **kw ):
 		apply( _PlotContainer.__init__, (self,) )
 		apply( self.conf_setattr, ("HammerAitoffPlot",), kw )
		self.content = _PlotComposite()
		self.l0 = l0
		self.b0 = b0
		self.rot = rot

	_attr_deprecated = {
		"num_l_ribs"	: "ribs_l",
		"num_b_ribs"	: "ribs_b",
	}

	def __setattr__( self, name, value ):
		self.__dict__[ self._attr_deprecated.get(name,name) ] = value

	def __iadd__( self, other ):
		self.add( other )

	def add( self, *args ):
		apply( self.content.add, args )

	def _draw_background( self, context ):
		pc = _PlotComposite()

		nl = self.ribs_l
		b = _series( -90/2, 90/2, 2*math.pi/180. )
		for l0 in _series( -nl, nl, math.pi/nl ):
			c = apply( Curve, ([l0]*len(b), b), self.ribs_style )
			pc.add( c )

		nb = self.ribs_b
		l = _series( -180/2, 180/2, 2*math.pi/180. )
		for b0 in _series( -nb, nb, 0.5*math.pi/(nb+1) ):
			c = apply( Curve, (l, [b0]*len(l)), self.ribs_style )
			pc.add( c )

		pc.render( context )

	def exterior( self, device, interior ):
		bb = interior.copy()
		context = _HammerAitoffContext( device, interior,
			self.l0, self.b0, self.rot )
		bb.union( self.content.bbox(context) )
		return bb

	def compose_interior( self, device, interior ):
		_PlotContainer.compose_interior( self, device, interior )
		context = _HammerAitoffContext( device, interior,
			self.l0, self.b0, self.rot )
		self._draw_background( context )
		self.content.render( context )




#
# $Id: confit.py,v 1.2 2001/04/26 04:12:15 mrnolta Exp $
#
# Copyright (C) 2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import ConfigParser
from string import atoi, split, strip

def _atox( x ):
	x = strip( x )
	if x == "None":
		return None
	try: return atoi( x, 0 )
	except ValueError:
		try: return float(x)
		except ValueError:
			pass
	if x[0] == "{" and x[-1] == "}":
		style = {}
		pairs = map( strip, split(x[1:-1], ",") )
		for pair in pairs:
			if pair == "":
				continue
			key,val = split( pair, ":" )
			style[ strip(key) ] = _atox( strip(val) )
		return style
	return x

class Confit:

	default_sect = "default"

	def __init__( self ):
		self._sections = {}
		self._deprecated = {}

	def _defaults( self ):
		return self._sections[ self.default_sect ]

	def _get( self, section, option ):
		return self._sections[section][option]

	def _set( self, section, option, value ):
		if not self._sections.has_key( section ):
			self._sections[section] = {}
		self._sections[section][option] = value

	def deprecated( self, old, new ):
		self._deprecated[old] = new

	def get( self, section, option, notfound=None ):
		try: rval = self._get( section, option )
		except KeyError:
			try: rval = self._defaults()[option]
			except KeyError:
				rval = notfound
		return rval

	def get_section( self, section ):
		if not self._sections.has_key( section ):
			return None
		return self._sections[section]

	def set( self, section, option, value ):
		if self._deprecated.has_key( (section,option) ):
			sect,opt = self._deprecated[ (section,option) ]
			self._set( sect, opt, value )
		else:
			self._set( section, option, value )

	def read( self, filename ):
		cp = ConfigParser.ConfigParser()
		cp.read( filename )

		for section in cp.sections():
			for option in cp.options( section ):
				if option == "__name__":
					continue
				a = cp.get( section, option, raw=1 )
				self.set( section, option, _atox(a) )




#
# $Id: config.py,v 1.34 2003/04/17 00:51:02 mrnolta Exp $
#
# Copyright (C) 2000-2001 Mike Nolta <mrnolta@users.sourceforge.net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#

import os, sys
import confit

_config = confit.Confit()

# XXX:deprecated 1.4
_config.deprecated( ('screen','guess_interactive'), ('screen','persistent') )

# XXX:deprecated 1.5
_config.deprecated( \
	('HammerAitoffPlot','num_b_ribs'), ('HammerAitoffPlot','ribs_b') )
_config.deprecated( \
	('HammerAitoffPlot','num_l_ribs'), ('HammerAitoffPlot','ribs_l') )
_config.deprecated( \
	("readcolumn","use_numeric"), ("read_column","return_numpy") )
_config.deprecated( \
	("readcolumn","comment_char"), ("read_column","comment_char") )

# XXX:deprecated 1.6.4
_config.deprecated( ('printer','paper'), ('postscript','paper') )

#SITECONFIGFILE = os.path.join( imp.find_module("biggles")[1], "config.ini" )
SITECONFIGFILE = os.path.join( os.path.dirname(__file__), "config.ini" )
_config.read( SITECONFIGFILE )

if os.environ.has_key( "HOME" ):
	USERCONFIGFILE = os.path.join( os.environ["HOME"], ".biggles" )
	_config.read( USERCONFIGFILE )

def interactive():
	return hasattr( sys, "ps1" )

def bool( section, option ):
	global _config
	try:
		x = _config.get( section, option, "no" )
		if x[0] == 'y':
			return 1
	except:
		return 0

def value( section, option, notfound=None ):
	global _config
	return _config.get( section, option, notfound )

def options( section ):
	global _config
	return _config.get_section( section )

def configure( *args ):
	global _config
	if len(args) == 2:
		_config.set( "default", args[0], args[1] )
	elif len(args) == 3:
		_config.set( args[0], args[1], args[2] )

